library(shiny)

shinyServer(function(input, output,session) {

    red <- paste("<font color=\"red\">")             
    blue <- paste("<font color=\"blue\">") 
    black <- paste("<font color=\"black\">") 
    plus1 <- paste("<font size=\"+1\">")
    
    data <- reactive({ 
         if(input$which=="Type I") x <- round(rnorm(input$n,10,3),1)
         if(input$which=="Type II") x <- round(rnorm(input$n,30,5),2)
         if(input$which=="Type III") x <- round(rnorm(input$n,0.5,0.1),3)
         if(input$which=="Type IIII") x <- round(rnorm(input$n,200,50))
         x
    })             

   breaks <- reactive({
          x <- data()
          k <- 1
          a <- as.numeric(input$left)
          repeat {
              k <- k+1
              a[k] <- as.numeric(input$left)+(k-1)*as.numeric(input$classwidth)    
              if(a[k]>max(x)) break
          }          
          a   
   })
   
  
   output$datatable1 <- renderText({
      x <- sort(data())
      
      a <- "<table>"
      k <- 0
      i <- 1
      repeat {
          i <- i+1
          a[i] <-"<Tr>"
          for(j in 1:10) {
              k <- k+1
              if(k>length(x)) break
              a[i] <- paste(a[i],"<Td><strong>",HTML("&nbsp;"),x[k]," </Td>",collapse="")
          }    
          a[i] <- paste(a[i],"</Tr>")
          if(k>length(x)) break
      }    
      a[i+1] <- "</table>"      

      a
   })  

   output$datatable2 <- renderText({
      x <- sort(data())
      brks <- breaks()
      leftedge <- brks
      rightedge <- brks[-1]-input$adj
      cols <- rep(black,length(x))
      isProblem <- F
      for(i in 1:length(x)) {
          for(j in 2:length(rightedge))
              if(rightedge[j-1]==x[i] & x[i]==leftedge[j]) {
              cols[i] <- red
              isProblem <- T
          }
      }
      if(isProblem) return(c("<h4>",paste(cols,x,collapse=""),"</h4>"))        
      for(i in 1:length(x)) {          
          for(j in 2:length(rightedge))
              if(rightedge[j-1]<x[i] & x[i]<leftedge[j]) cols[i] <- red
      }
      c("<h4>",paste(cols,x,collapse=""),"</h4>")        
          
      
   })
   
   output$text1 <- renderText({
          if(input$left=="") return(NULL) 
          if(input$classwidth=="") return(NULL)  
          if(as.numeric(input$left)>min(data())) return("")                       
          brks <- breaks()
          counts <- hist(data(),breaks=brks,plot=F)$counts
          tbl <- "<table border=\"1\" >"
          tbl[2] <- paste("<TR><TH>",plus1,"Classes</TH><TH>",plus1,"Frequencies</TH></TR>",sep="")
          for(i in 1:(length(brks)-1)) {
              tbl[i+2] <-paste("<Tr><Td align=\"right\">",plus1,brks[i],"-",brks[i+1]-input$adj,"</Td><TD align=\"right\">",plus1,counts[i],"</TD></TR>")
          }    
          c(tbl,"</table>")            
   })        

   output$text2 <- renderText({
          x <- data()
          line <- paste("<h4>Minimum=",min(x),"    Maximum=",max(x),"</h4>")
          if(input$classwidth=="") return(c(line,"<h4>Choose classwith</h4>"))          
          line[2] <- paste("<h4>Classwidth =",input$classwidth,"</h4>")
          if(input$left=="") return(c(line,"<h4>Choose left endpoint!</h4>"))
          line[3] <- paste("<h4>Left endpoint =",input$left,"</h4>")
          line
      
   }) 
   
   output$text3 <- renderText({   
             if(input$left=="") return(NULL) 
             if(input$classwidth=="") return(NULL)       
             x <- data()
             brks <- breaks()
             if(as.numeric(input$left)>min(data())) return(paste("<h3>",red,"Left Endpoint is larger than Minimum!</h3>"))             
             counts <- hist(data(),breaks=brks,plot=F)$counts
             if(counts[1]==0) return(paste("<h3>",red,"Count in first class is 0!</h3>"))          
             leftedge <- brks[-1]
             rightedge <- brks[-1]-input$adj
             if(input$adj==0) {
              for(i in 1:length(rightedge)) {
                z1 <- c(1:length(x))[x==leftedge[i]]
                if(length(z1)>0) {
                    z2 <- c(1:length(x))[x==rightedge[i]]
                    if(length(z2)>0) {
                        if(max(z1 %in% z2)==1)   
                           return(paste("<h3>",red,"Some Observations in two classes!</h3>"))          
                    }    
                }    
              }
             } 
             for(i in 1:length(rightedge)) {
                z <- x[x>rightedge[i]]
                z <- z[z<leftedge[i]]
                if(length(z)>0) {
                    print("AAA")
                    return(paste("<h3>",red,"Some Observations not in any class!</h3>"))          
                }    
             } 
             ""
   })
    
   output$plot1 <- renderPlot({
          if(input$left=="") return(NULL)   
          if(input$classwidth=="") return(NULL)           
          if(as.numeric(input$left)>min(data())) return("")             
          hist(data(),breaks=breaks(),xlab="Data",main="")
   })
           

  output$explainText <- renderText({ 
        "<h5>Explanation: This demo illustrates the how to do a Frequency table and then draw a histogram</h5>"
        
  })

})

